using Infrastructure.Options;
using Microsoft.IdentityModel.Tokens;

namespace Infrastructure.Extensions;

public static class AuthorizeSetup
{
    public static IServiceCollection AddDefaultAuthorize(this IServiceCollection services, IConfiguration configuration)
    {
        ArgumentNullException.ThrowIfNull(services);
        ArgumentNullException.ThrowIfNull(configuration);
        var audienceOptions = configuration.GetSection(AudienceOptions.Name).Get<AudienceOptions>();
        if (audienceOptions is null || !audienceOptions.IsEnable)
        {
            return services;
        }

        var key = configuration["AUDIENCE_KEY"] ?? audienceOptions.Secret;
        ArgumentException.ThrowIfNullOrEmpty(key);
        var buffer = Encoding.UTF8.GetBytes(key);
        var securityKey = new SymmetricSecurityKey(buffer);
        var signingCredentials = new SigningCredentials(securityKey, SecurityAlgorithms.HmacSha256);

        services.AddSingleton(new JwtOptions(
            audienceOptions.Issuer,
            audienceOptions.Audience,
            audienceOptions.Expiration,
            signingCredentials));

        services.AddAuthorizationBuilder()
            .AddPolicy(audienceOptions.Policy!, policy =>
                policy.RequireRole(audienceOptions.Roles!)
                    .Build());
        return services;
    }
}